%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%THIS CODE GENERATES VARIABLES FOR FIGURE 4 IN THE PAPER: 
% INFERENCE FOR VARs IDENTIFIED WITH SIGN RESTRICTIONS By Eleonora Granziera
% Hyungsik Roger Moon and Frank Schorfheide (2017)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Housekeeping
clear all
rng('default')

alpha1  = 0.05;         % overall significance level
testrun = 1;           % if testrun = 1 we use small number of repetitions
                       % if testrun = 0 we use large number of repetitions
nobs = 170;            % sample size
nirfsign = 2;          % horizon for IRF restrictions
                       
% set up directory structure
paths.prog = pwd; %get current directory

                                 
%% Data Generating Process
parameters = dlmread('DGP.txt'); 

% Dimension of VAR
nvar  = size(parameters,2); % # of variables in the var
nlags = floor((size(parameters,1)-nvar)/nvar); % # of lags 
if size(parameters,1) == nvar*nlags + nvar;
    intercept = 0;
else
    intercept = 1;
end

% VAR parameterization
% Phi is of dimention (nvar*nlags+intercept)x (nvar)
Phi_dgp   = parameters(1:nvar*nlags+intercept,:); %Parameters are [A1';A2';c']; 
% Sigma is of dimention (nvar) x (nvar)
Sigma_dgp = parameters(nvar*nlags+intercept + 1:end,:); 


%% Configuration of Monte Carlo
if testrun == 1
    nsim        = 50;   % # of Monte Carlo repetitions
    nobsinitial = 20;   % # of initial draws for VAR simulation
    nsim_lambda = 1E2;  % # of simulations to estimate lambda
    nsim_cv     = 1E3;  % # of simulations to approx critical values 
    nq          = 1E3;  % size of grid for q
else
    nsim        = 500;  % # of Monte Carlo repetitions
    nobsinitial = 50;   % # of initial draws for VAR simulation
    nsim_lambda = 1E3;  % # of simulations to estimate lambda
    nsim_cv     = 1E3;  % # of simulations to approx critical values 
    nq          = 2*1E4;  % size of grid for q
end

nirf  = 24;    % maximum horizon for IRFs

%% Sign restrictions
%   endogenous variables are: 
%         output, inflation, interest rates, real money
%   restricted responses: 
%         inflation <= 0; interest rates >=0; money <=0; 

signrestrictions      = zeros(nirfsign,nvar);
signrestrictions(:,2) = -ones(nirfsign,1);
signrestrictions(:,3) =  ones(nirfsign,1);
signrestrictions(:,4) = -ones(nirfsign,1);

% Construct summary of sign restrictions
NumberSignRestr      = sum(sum(abs(signrestrictions)));
signrestrictionsindx = zeros(NumberSignRestr,3); 
count = 0;
for i = 1: nirfsign;
  for j =1: nvar;
      if signrestrictions(i,j) ~= 0;
         count=count+1;
         % first column says which variable is restricted for each lag
         signrestrictionsindx(count,1) = j; 
         % second column tells you which lag
         signrestrictionsindx(count,2) = i; 
         % third column tells you which restriction is imposed
         signrestrictionsindx(count,3) = signrestrictions(i,j);  
      end;
  end;
end;

%% Generate grid for q
rng(1); % seeds the random number generator
q_grid = gridq(nvar,nq); 


%% Generate asymptotic covariance matrix LAMBDA
rng(1); % seeds random number generator 
[Lambda_all, Lambda_restricted, psi_mean] = estimate_lambda(Phi_dgp, Sigma_dgp, ...
                            nsim_lambda, nirf, nobs, nobsinitial, signrestrictionsindx);

                        
%% Compute identified set for q and theta  %%%%%%%%%%%%%%%%%%%
disp('************************');
disp('Computing Identified Set for q  and theta');
disp('                        ');

[ YYirf_dgp ]                = construct_rfirf(Phi_dgp, Sigma_dgp, nirf);
[ varirf_id_l, varirf_id_u, q_id, q_id_index , GTqpop] = construct_idset(q_grid, ...
          signrestrictionsindx, signrestrictions, nirf, YYirf_dgp);
length_id = varirf_id_u - varirf_id_l;


clear parameters YYirf


for alpha2 = [0.05  0.1  0.15] % significance level for q confidence set

alpha = alpha1 + alpha2; % significance level for theta confidence set given q
    
paths.results.main = [ paths.prog '\Nobs' num2str(nobs) ...
                       'RestrHorizon' num2str(nirfsign) 'Alpha' num2str(100*alpha) ];


paths.results.sub  = [ paths.results.main '\AlphaOne' num2str(100*alpha1) ...
                       'AlphaTwo' num2str(100*alpha2) ];

mkdir(paths.results.sub); 

q_coveAS = zeros(1,nq);

n_bind_aveAS = zeros(1,nq);
GT_ave = zeros(1,nq);

varirf_uAS = zeros(nirf,nvar);
varirf_lAS = zeros(nirf,nvar);
theta_coveuAS = zeros(nirf,nvar);
theta_covelAS = zeros(nirf,nvar);
length_csAS = zeros(nirf,nvar);

cd(paths.prog); %get current directory

%% MONTE CARLO ANALYSIS
for ss = 1:nsim
    
    fprintf('Iteration = %d\n',ss);
    tic
    
    %%% STEP 1.1: generate artificial data
    rng(ss);
    [YY] = generate_data( Phi_dgp, Sigma_dgp, nobs, nobsinitial);
    
    %%% STEP 1.2: estimate VAR(p)
    [phi_hat, sigma_hat] = estimate_rfvar( YY, nlags, intercept );
    
    %%% STEP 1.3: estimate VAR(p) and generate IRF
    % Yirf:
    % h=1: (sh1 -> v1) (sh2 -> v1) (sh3 -> v1) (sh1 -> v2) (sh2 -> v2) (sh3 -> v2)
    % h=2: (sh1 -> v1) (sh2 -> v1) (sh3 -> v1) (sh1 -> v2) (sh2 -> v2) (sh3 -> v2)
    [ YYirf_hat ] = construct_rfirf(phi_hat, sigma_hat, nirf);
    
    % column vector of dimention nvar^2 x nirf; includes
    % all responses of all variables to all orthogonalized shocks;
    % vectorization of  YYirf
    psi_hat_all = reshape(YYirf_hat',nvar*nvar*nirf,1);
    
    % select the IRF that are restricted (variables and horizons)
    % in the MC of the paper, YYirfselect  =
    % [h=1 (sh1 -> v2) (sh2 -> v2) (sh3 -> v2) (sh4 -> v2);
    % h=1 (sh1 -> v3) (sh2 -> v3) (sh3 -> v3) (sh4 -> v3);
    % h=2 (sh1 -> v2) (sh2 -> v2) (sh3 -> v2) (sh4 -> v2);
    % h=2 (sh1 -> v3) (sh2 -> v3) (sh3 -> v3) (sh4 -> v3)]
    YYirf_restricted_hat = select_irf(YYirf_hat,signrestrictionsindx);
    
    % YYirfselect once zero elements have been dropped.
    % In the MC example psi_hat =
    % [(h=1 sh1 -> v2) (h=1 sh2 -> v2)
    %  (h=1 sh1 -> v3) (h=1 sh2 -> v3) (h=1 sh3 -> v3)
    %  (h=1 sh1 -> v4) (h=1 sh2 -> v4) (h=1 sh3 -> v4) (h=1 sh4 -> v4)
    %  (h=2 sh1 -> v2) (h=2 sh2 -> v2) (h=2 sh3 -> v2) (h=2 sh4 -> v2)
    %   ....(h=2 sh4 -> v4)]';
    [ psi_hat_restricted, index_phi ] = construct_psi_restricted(YYirf_restricted_hat);
    
    
    %%%% STEP 1.4: compute confidence set for q
    [ temp_qAS,  temp_bind,  cvAS, temp_GT] = construct_qcs(q_grid, Lambda_restricted, psi_hat_restricted, signrestrictionsindx, index_phi, nobs, nsim_cv, alpha1);
    
    q_coveAS = q_coveAS + temp_qAS/nsim;
    n_bind_aveAS = n_bind_aveAS + temp_bind/nsim;
    GT_ave = GT_ave + temp_GT/nsim;
    
    %%%% STEP 1.5: compute Wald confidence sets for responses
    
    temp_qset = temp_qAS.*[1:1:size(temp_qAS,2)];
    sel = temp_qset(temp_qset~=0);
    qset_tempAS = q_grid(:,sel);
    
    [ temp_lAS, temp_uAS ] = construct_irfcs( qset_tempAS, YYirf_hat, Lambda_all, nlags, nirf, nobs , signrestrictions, nirfsign, alpha2);
    
    varirf_uAS = varirf_uAS + temp_uAS/nsim;
    varirf_lAS = varirf_lAS + temp_lAS/nsim;
    
    theta_coveuAS = theta_coveuAS +(( temp_uAS - varirf_id_u ) >=  0)/nsim;
    theta_covelAS = theta_covelAS + (( temp_lAS - varirf_id_l ) <=  0.001)/nsim;
    length_csAS = length_csAS + ( temp_uAS - temp_lAS )/nsim;
    
    clear temp_qset sel qset_tempAS
    
    disp(['Elapsed Time: ', num2str(toc)]);

end %end of loop over random samples of data


%Save Workspace
cd(paths.results.sub);
save ( ['WorkspaceNobs' num2str(nobs) 'Alpha' num2str(100*alpha) ])


end %end of loops over values of alpha2