%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Replication Files for Empirical Application (Figure 5 and 6) in:
% 
% INFERENCE FOR VARs IDENTIFIED WITH SIGN RESTRICTIONS
%
% by Eleonora Granziera, Hyungsik Roger Moon and Frank Schorfheide (2017)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all
rng('default')

testrun = 0;   % if testrun = 1 we use small number of repetitions
               % if testrun = 0 we use large number of repetitions
nobs = 165;    % sample size
nirfsign = 2;  % horizon for IRF restrictions; nirfsign = 2 ==> restrict IRFs on impact and 
                                               % one quarter after the shock

% Set up directory 
paths.prog = pwd; %get current directory

%% Read Data 
data = dlmread('datampshock.txt');
data(:,4) = data(:,1) + data(:,4); %convert velocity into real money balances

% Dimension of VAR
nvar  = size(data,2); % # of variables in the var
nlags = 2; % # of lags
intercept = 1;


% % VAR parameterization
% % Phi is of dimension (nvar*nlags+intercept)x (nvar)
% Phi_ols   = parameters(1:nvar*nlags+intercept,:); %Parameters are [A1';A2';c'];
% % Sigma is of dimension (nvar) x (nvar)
% Sigma_ols = parameters(nvar*nlags+intercept + 1:end,:);

[ Phi_ols, Sigma_ols ] = estimate_rfvar( data(3:end,:), nlags, intercept );

clear parameters

%% Configuration of Simulations for Estimators
if testrun == 1
    ndraws      = 1000; % #draws from posterior
    ndrawsout   = 100;  % plot output every ndrawsout draws
    nobsinitial = 20;   % # of initial draws for VAR simulation in procedure to estimate lambda
    nsim_lambda = 1E2;  % # of simulations to estimate lambda
    nsim_cv     = 1E3;  % # of simulations to approx critical values
    nq          = 1E3;  % size of grid for q
else
    ndraws      = 50000; % #draws from posterior
    ndrawsout   = 1000;  % plot output every ndrawsout draws    
    nobsinitial = 50;   % # of initial draws for VAR simulation
    nsim_lambda = 1E3;  % # of simulations to estimate lambda
    nsim_cv     = 1E3;  % # of simulations to approx critical values
    nq          = 2*1E4;  % size of grid for q
end

nirf   = 24 ;  % maximum horizon for IRFs
alpha1 = 0.05; % significance level for q confidence set
alpha  = 0.10; % desired significance level for theta set

%% Sign restrictions
%   endogenous variables are:
%         output, inflation, interest rates, real money
%   restricted responses:
%         inflation <= 0; interest rates >=0; money <=0;

signrestrictions      = zeros(nirfsign,nvar);
signrestrictions(:,2) = -ones(nirfsign,1);
signrestrictions(:,3) =  ones(nirfsign,1);
signrestrictions(:,4) = -ones(nirfsign,1);

% Construct summary of sign restrictions
NumberSignRestr      = sum(sum(abs(signrestrictions)));
signrestrictionsindx = zeros(NumberSignRestr,3);
count = 0;
for i = 1: nirfsign;
    for j =1: nvar;
        if signrestrictions(i,j) ~= 0;
            count=count+1;
            % first column says which variable is restricted for each lag
            signrestrictionsindx(count,1) = j;
            % second column tells you which lag
            signrestrictionsindx(count,2) = i;
            % third column tells you which restriction is imposed
            signrestrictionsindx(count,3) = signrestrictions(i,j);
        end;
    end;
end;

clear count

%% Generate grid for q
rng(1); % seeds the random number generator
q_grid = gridq(nvar,nq);


%% Generate asymptotic covariance matrix LAMBDA
rng(1); % seeds random number generator
[Lambda_all, Lambda_restricted, psi_mean] = estimate_lambda(Phi_ols, Sigma_ols, ...
    nsim_lambda, nirf, nobs, nobsinitial, signrestrictionsindx);


%% Compute identified set for q and theta
disp('************************');
disp('Computing Identified Set');
disp('                        ');
[ YYirf_hat ] = construct_rfirf(Phi_ols, Sigma_ols, nirf);
[ varirf_id_l, varirf_id_u, q_id, q_id_index , GTqpop] = construct_idset(q_grid, ...
    signrestrictionsindx, signrestrictions, nirf, YYirf_hat);
length_id = varirf_id_u - varirf_id_l;


%% Compute Bayesian credible sets
disp('******************************');
disp('Computing Bayesian Cred. Bands');
disp('                              ');

nvalid = 0; %number of draws that satisfy the resrictions
XX = construct_regressors(data, nlags, intercept);

while nvalid < ndraws 
    % sample from posterior distribution and construct irf
    Sigma_new  =  iwishrnd(nobs*Sigma_ols,nobs-nvar*nlags+1);
    phi_new    =  mvnrnd(reshape(Phi_ols,size(Phi_ols,1)*size(Phi_ols,2),1),kron(Sigma_new,inv(XX'*XX)));
    [ YYirf_new, valid ] = construct_rfirfsign(reshape(phi_new,nvar*nlags+intercept,nvar), Sigma_new, nirf, signrestrictionsindx);

    if valid == 1
        nvalid = nvalid+1;
        YYirf_valid(:,nvalid) =  YYirf_new;
        %stack the responses: YYirf_valid is 
        %[var1h1;...;var1h24;var2h1;...var2h24;....var4h24]
        
        if mod(nvalid,ndrawsout) == 0
           disp(['Number of Draws Processed ', num2str(nvalid), ' out of ', num2str(ndraws)]);
        end

    end
        
end

% construct 90% HPDI
hpdilow = zeros(nirf, nvar);
hpdihigh = zeros(nirf, nvar);

for i = 1:nvar
    [hpdilow(:,i), hpdihigh(:,i)] = HPDI(YYirf_valid((i-1)*nirf+1:i*nirf,:)',alpha);
end

%% Compute frequentist confidence sets and plot figures

disp('**************************');
disp('Computing Freq Conf. Bands');
disp('                          ');

% column vector of dimention nvar^2 x nirf; includes
% all responses of all variables to all orthogonalized shocks;
% vectorization of  YYirf
psi_hat_all = reshape(YYirf_hat',nvar*nvar*nirf,1);

% select the IRF that are restricted (variables and horizons)
YYirf_restricted_hat = select_irf(YYirf_hat,signrestrictionsindx);

% YYirfselect once zero elements have been dropped.
[ psi_hat_restricted, index_phi ] = construct_psi_restricted(YYirf_restricted_hat);

% Iterate over choices of alpha2. The Bonferroni set is a 1-alpha
% confidence set if alpha = alpha1 + alpha2

for alpha2 = [0.05  0.10]
    
    disp('-----------------------------');
    disp(['Alpha2 = ' num2str(alpha2) ]);
    disp('                          ');
    
    paths.results.sub  = [ paths.prog '\PureSignNobs' num2str(nobs) ...
        'RestrHorizon' num2str(nirfsign) 'Alpha' num2str(100*alpha) '\AlphaOne' num2str(100*alpha1) ...
        'AlphaTwo' num2str(100*alpha2) ];
    
    mkdir(paths.results.sub);
    
    cd(paths.prog); %get current directory
    
    % STEP 1: compute confidence set for q
    disp('Computing Confidence Set for q');
    [ q_binary,  n_bind_aveAS,  cvAS, GT_ave] = construct_qcs(q_grid, Lambda_restricted, psi_hat_restricted, signrestrictionsindx, index_phi, nobs, nsim_cv, alpha1);
    
    
    % STEP 2: compute conditional CS for theta and take union
    disp('Computing Confidence Sets for IRFs');
    temp_qset = q_binary.*[1:1:size(q_binary,2)];
    sel = temp_qset(temp_qset~=0);
    qset = q_grid(:,sel);
    clear temp_qset sel
    [ varirf_lAS, varirf_uAS ] = construct_irfcs( qset, YYirf_hat, Lambda_all, nlags, nirf, nobs , signrestrictions, nirfsign, alpha2);
    length_csAS = varirf_uAS - varirf_lAS;
        
    % SAVE AND PLOT FIGURES
    cd(paths.results.sub);
    x_horizon = [0:1:nirf-1];
    save ( ['WorkspaceNobs' num2str(nobs) 'Alpha' num2str(100*alpha) ...
            'AlphaOne' num2str(100*alpha1) 'AlphaTwo' num2str(100*alpha2)], ...
            'x_horizon', 'varirf_id_l', 'varirf_lAS', 'hpdilow', ...
                       'varirf_id_u', 'varirf_uAS', 'hpdihigh') ;
    
    % Output         
    fig = figure('Name','Output: ID Set (Black), Freq (Blue), Bayes (Red)');
        fig2_var1cs = plot( x_horizon, varirf_id_l(:,1) , 'k-x', x_horizon, varirf_lAS(:,1), 'b', x_horizon, hpdilow(:,1),'r-o', x_horizon, varirf_id_u(:,1) , 'k--x', ...
        x_horizon, varirf_uAS(:,1), 'b--', x_horizon, hpdihigh(:,1),'r--o', x_horizon, zeros(nirf,1),'k' , 'LineWidth',3);
        xlim([0 nirf-1]);
        set(gca,'FontSize',28,'FontWeight','Bold')
        savefilename = [ 'var1cs.pdf' ];
        saveas(fig, savefilename);
    
    % Inflation
    fig = figure('Name','Inflation: ID Set (Black), Freq (Blue), Bayes (Red)');
        fig2_var12cs =  plot( x_horizon, varirf_id_l(:,2) , 'k-x', x_horizon, varirf_lAS(:,2), 'b', x_horizon, hpdilow(:,2),'r-o', x_horizon, varirf_id_u(:,2) , 'k--x', ...
        x_horizon, varirf_uAS(:,2), 'b--',x_horizon, hpdihigh(:,2),'r--o', x_horizon, zeros(nirf,1),'k' , 'LineWidth',3);
        xlim([0 nirf-1]);
        set(gca,'FontSize',28,'FontWeight','Bold')
        savefilename = [ 'var2cs.pdf' ];
        saveas(fig, savefilename);
    
    % Interest Rates
    fig = figure('Name','Interest Rate: ID Set (Black), Freq (Blue), Bayes (Red)');
        fig2_var3cs = plot( x_horizon, varirf_id_l(:,3) , 'k-x', x_horizon, varirf_lAS(:,3), 'b', x_horizon, hpdilow(:,3),'r-o',  x_horizon, varirf_id_u(:,3) , 'k--x', ...
        x_horizon, varirf_uAS(:,3), 'b--',x_horizon, hpdihigh(:,3),'r--o', x_horizon, zeros(nirf,1),'k' , 'LineWidth',3);
        xlim([0 nirf-1]);
        set(gca,'FontSize',28,'FontWeight','Bold')
        savefilename = [ 'var3cs.pdf' ];
        saveas(fig, savefilename);
    
    % Money
    fig = figure('Name','Money: ID Set (Black), Freq (Blue), Bayes (Red)');
        fig2_var14cs =  plot( x_horizon, varirf_id_l(:,4) , 'k-x', x_horizon, varirf_lAS(:,4), 'b', x_horizon, hpdilow(:,4),'r-o', x_horizon, varirf_id_u(:,4) , 'k--x', ...
        x_horizon, varirf_uAS(:,4), 'b--',x_horizon, hpdihigh(:,4),'r--o' , x_horizon, zeros(nirf,1),'k' , 'LineWidth',3);
        xlim([0 nirf-1]);
        set(gca,'FontSize',28,'FontWeight','Bold')
        savefilename = [ 'var4cs.pdf' ];
        saveas(fig, savefilename);
    
end %end of loops for values of alpha2